::cisco::eem::event_register_timer watchdog time $high_cpu_poll_freq name "high_cpu_timer"
#-
# Copyright (c) 2010 Joe Marcus Clarke <jclarke@cisco.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#
# This policy polls the five second CPU utilization of a given CPU, and checks
# to see if it crosses a threshold.  If so, then it will extract the top
# processes from "show proc cpu" and send those processes as well as their
# five second CPU time out via syslog message.
#
# This policy takes the following environment variable(s):
#
# high_cpu_threshold  : High CPU threshold.  This must be a number between 0
#                       and 100.
#
# high_cpu_poll_freq  : The number of seconds between each polling interval.
#
#

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

if [catch {cli_open} result] {
    error $result $errorInfo
} else {
    array set cli1 $result
}

if [catch {cli_exec $cli1(fd) "enable"} _cli_result] {
    error $_cli_result $errorInfo
}

if [catch {cli_exec $cli1(fd) "show proc cpu sorted"} _cli_result] {
    error $_cli_result $errorInfo
}

array set util [list]
set plist [list]
foreach line [split $_cli_result "\n"] {
    if { ! [regexp {(\d+\.\d\d)%\s+\d+\.\d\d%\s+\d+\.\d\d%\s+\d+\s+(.+)} $line -> perc pname] } {
	continue
    }

    if { $perc >= $high_cpu_threshold } {
	lappend plist $pname
	set util($pname) $perc
    } else {
	break
    }
}

if { [llength $plist] == 0 } {
    exit 0
}

set msg "HIGH CPU: top processes: "
foreach pname $plist {
    append msg "[string trim $pname] ($util($pname)%) "
}

action_syslog priority crit msg $msg

# Close open cli before exit.
if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
    error $result $errorInfo
}
